@@ -1385,7 +1385,7 @@ static uint32_t _tpm20_shutdown(uint32_t locality, u16 type)
     return ret;
 }
 
-static u32 handle2048;
+static __data u32 handle2048 = 0;
 static const char auth_str[] = "test";
 static uint32_t _tpm20_create_primary(uint32_t locality,
                                      tpm_create_primary_in *in,
@@ -1646,10 +1646,8 @@ static bool tpm20_pcr_read(struct tpm_if *ti, uint32_t locality,
     tpm_pcr_read_out read_out;
     u32 ret;
 
-    if ( ti == NULL || out == NULL ){
-        ti->error = TPM_RC_FAILURE;
+    if ( ti == NULL || out == NULL )
         return false;
-    }
 
     read_in.pcr_selection.count = 1;
     read_in.pcr_selection.selections[0].hash = ti->cur_alg;
@@ -1680,10 +1678,8 @@ static bool tpm20_pcr_extend(struct tpm_if *ti, uint32_t locality,
     tpm_pcr_extend_out extend_out;
     u32 ret, i;
 
-    if ( ti == NULL || in == NULL ){
-        ti->error = TPM_RC_FAILURE;
+    if ( ti == NULL || in == NULL )
         return false;
-    }
 
     extend_in.pcr_handle = pcr;
     extend_in.sessions.num_sessions = 1;
@@ -1738,10 +1734,8 @@ static bool tpm20_hash(struct tpm_if *ti, u32 locality, const u8 *data,
     TPM2B_MAX_BUFFER buffer;
     u32 ret, i, j, chunk_size;
 
-    if ( ti == NULL || data == NULL || data_size == 0 ) {
-        ti->error = TPM_RC_FAILURE;
+    if ( ti == NULL || data == NULL || data_size == 0 )
         return false;
-    }
 
     start_in.auth.t.size = 2;
     start_in.auth.t.buffer[0] = 0;
@@ -1814,10 +1808,9 @@ static bool tpm20_nv_read(struct tpm_if *ti, uint32_t locality,
     tpm_nv_read_out read_out;
     u32 ret;
 
-    if ( ti == NULL || data_size == NULL || *data_size == 0 ) {
-        ti->error = TPM_RC_FAILURE;
+    if ( ti == NULL || data_size == NULL || *data_size == 0 )
         return false;
-    }
+
     if ( *data_size > MAX_NV_INDEX_SIZE )
         *data_size = MAX_NV_INDEX_SIZE;
 
@@ -1852,10 +1845,8 @@ static bool tpm20_nv_write(struct tpm_if *ti, uint32_t locality,
     u32 ret;
 
     if ( ti == NULL || data == NULL || data_size == 0 
-            || data_size == 0 || data_size > MAX_NV_INDEX_SIZE ) {
-        ti->error = TPM_RC_FAILURE;
+            || data_size > MAX_NV_INDEX_SIZE )
         return false;
-    }
 
     write_in.handle = index;
     write_in.index = index;
@@ -1883,10 +1874,8 @@ static bool tpm20_get_nvindex_size(struct tpm_if *ti, uint32_t locality,
     tpm_nv_read_public_out public_out;
     u32 ret;
 
-    if ( ti == NULL || size == NULL ) {
-        ti->error = TPM_RC_FAILURE;
+    if ( ti == NULL || size == NULL )
         return false;
-    }
 
     public_in.index = index;
 
@@ -1913,10 +1902,8 @@ static bool tpm20_get_nvindex_permission(struct tpm_if *ti, uint32_t locality,
                                     uint32_t index, uint32_t *attribute)
 {
     if ( ti == NULL || locality >= TPM_NR_LOCALITIES
-         || index == 0 || attribute == NULL ) {
-        ti->error = TPM_RC_FAILURE;
+         || index == 0 || attribute == NULL )
         return false;
-    }
 
     return true;
 }
@@ -1941,6 +1928,7 @@ static bool tpm20_seal(struct tpm_if *ti, uint32_t locality,
     create_in.public.t.public_area.auth_policy.t.size = 0;
     *(u32 *)&create_in.public.t.public_area.object_attr = 0;
     create_in.public.t.public_area.object_attr.userWithAuth = 1;
+    create_in.public.t.public_area.object_attr.noDA = 1;
     create_in.public.t.public_area.param.keyed_hash.scheme.scheme = TPM_ALG_NULL;
     create_in.public.t.public_area.unique.keyed_hash.t.size = 0;
 
@@ -1977,10 +1965,8 @@ static bool tpm20_unseal(struct tpm_if *ti, uint32_t locality,
     u32 ret;
 
     if ( ti == NULL || locality >= TPM_NR_LOCALITIES
-         || sealed_data_size == 0 || sealed_data == NULL ) {
-        ti->error = TPM_RC_FAILURE;
+         || sealed_data_size == 0 || sealed_data == NULL )
         return false;
-    }
 
     /* For TPM 2.0, the object will need to be loaded before it may be used.*/
     load_in.parent_handle = handle2048;
@@ -2022,10 +2008,8 @@ static bool tpm20_unseal(struct tpm_if *ti, uint32_t locality,
 static bool tpm20_verify_creation(struct tpm_if *ti, uint32_t sealed_data_size,
                                   uint8_t *sealed_data)
 {
-    if ( ti == NULL || sealed_data_size == 0 || sealed_data == NULL ) {
-        ti->error = TPM_RC_FAILURE;
+    if ( ti == NULL || sealed_data_size == 0 || sealed_data == NULL )
         return false;
-    }
 
     return true;
 }
@@ -2038,10 +2022,8 @@ static bool tpm20_get_random(struct tpm_if *ti, uint32_t locality,
     u32 ret, out_size, requested_size;
     static bool first_attempt;
 
-    if ( random_data == NULL || data_size == NULL || *data_size == 0 ) {
-        ti->error = TPM_RC_FAILURE;
+    if ( random_data == NULL || data_size == NULL || *data_size == 0 )
         return false;
-    }
 
     first_attempt = true;
     requested_size = *data_size;
@@ -2095,6 +2077,9 @@ static uint32_t tpm20_save_state(struct tpm_if *ti, uint32_t locality)
 {
     u32 ret;
 
+    if ( ti == NULL )
+        return false;
+
     ret = _tpm20_shutdown(locality, TPM_SU_STATE);
     if ( ret != TPM_RC_SUCCESS ) {
         printk(TBOOT_WARN"TPM: Shutdown, return value = %08X\n", ret);
@@ -2104,13 +2089,41 @@ static uint32_t tpm20_save_state(struct tpm_if *ti, uint32_t locality)
     return ret;
 }
 
+#define TPM_NR_PCRS             24
+#define TPM_PCR_RESETABLE_MIN   16
 static bool tpm20_cap_pcrs(struct tpm_if *ti, u32 locality, int pcr)
 {
-    if ( ti == NULL || locality >= TPM_NR_LOCALITIES || pcr == 0 ) {
-        ti->error = TPM_RC_FAILURE;
+    bool was_capped[TPM_NR_PCRS] = {false};
+    hash_list_t cap_val;   /* use whatever val is on stack */
+
+    if ( ti == NULL || locality >= TPM_NR_LOCALITIES || pcr == 0 )
         return false;
+
+    cap_val.count = ti->banks;
+    for (unsigned int i=0; i<ti->banks; i++)
+        cap_val.entries[i].alg = ti->algs_banks[i];
+
+    if (pcr >= 0) {
+        tpm20_pcr_extend(ti, locality, pcr, &cap_val);
+        return true;
     }
 
+    /* ensure PCRs 17 + 18 are always capped */
+    tpm20_pcr_extend(ti, locality, 17, &cap_val);
+    tpm20_pcr_extend(ti, locality, 18, &cap_val);
+    was_capped[17] = was_capped[18] = true;
+
+    /* also cap every dynamic PCR we extended (only once) */
+    /* don't cap static PCRs since then they would be wrong after S3 resume */
+    memset(&was_capped, true, TPM_PCR_RESETABLE_MIN*sizeof(bool));
+    for ( int i = 0; i < g_pre_k_s3_state.num_vl_entries; i++ ) {
+        if ( !was_capped[g_pre_k_s3_state.vl_entries[i].pcr] ) {
+            tpm20_pcr_extend(ti, locality, g_pre_k_s3_state.vl_entries[i].pcr, &cap_val);
+            was_capped[g_pre_k_s3_state.vl_entries[i].pcr] = true;
+        }
+    }
+
+    printk(TBOOT_INFO"cap'ed dynamic PCRs\n");
     return true;
 }
 
@@ -2126,9 +2139,12 @@ static bool alg_is_supported(u16 alg)
 
 static bool tpm20_init(struct tpm_if *ti)
 {
-    u32 ret, count=0;
+    u32 ret;
     unsigned int i;
 
+    if ( ti == NULL )
+        return false;
+
     /* init version */
     ti->major = TPM20_VER_MAJOR;
     ti->minor = TPM20_VER_MINOR;
@@ -2182,10 +2198,13 @@ static bool tpm20_init(struct tpm_if *ti)
             ti->alg_count++;
         }
     }
-    printk(TBOOT_INFO"TPM: supported alg cout = %08X\n", count);
+    printk(TBOOT_INFO"TPM: supported alg cout = %08X\n", ti->alg_count);
     for (unsigned int i=0; i<ti->alg_count; i++)
         printk(TBOOT_INFO"\t\t %08X\n", ti->algs[i]);
 
+    if (handle2048 != 0)
+        goto out;
+
     /* create primary object as parent obj for seal */
     tpm_create_primary_in primary_in;
     tpm_create_primary_out primary_out;
@@ -2209,11 +2228,12 @@ static bool tpm20_init(struct tpm_if *ti)
     primary_in.public.t.public_area.object_attr.decrypt = 1;
     primary_in.public.t.public_area.object_attr.fixedTPM = 1;
     primary_in.public.t.public_area.object_attr.fixedParent = 1;
+    primary_in.public.t.public_area.object_attr.noDA = 1;
     primary_in.public.t.public_area.object_attr.sensitiveDataOrigin = 1;
     primary_in.public.t.public_area.auth_policy.t.size = 0;
     primary_in.public.t.public_area.param.rsa.symmetric.alg = TPM_ALG_AES;
     primary_in.public.t.public_area.param.rsa.symmetric.key_bits.aes= 128;
-    primary_in.public.t.public_area.param.rsa.symmetric.mode.aes = TPM_ALG_ECB;
+    primary_in.public.t.public_area.param.rsa.symmetric.mode.aes = TPM_ALG_CFB;
     primary_in.public.t.public_area.param.rsa.scheme.scheme = TPM_ALG_NULL;
     primary_in.public.t.public_area.param.rsa.key_bits = 2048;
     primary_in.public.t.public_area.param.rsa.exponent = 0;
@@ -2229,8 +2249,8 @@ static bool tpm20_init(struct tpm_if *ti)
     }
     handle2048 = primary_out.obj_handle;
  
+out:
     tpm_print(ti);
-
     return true;
 }
 
